
#include "unix.h"
#include "x.h"
#include "xt.h"
#include "xmu.h"
#include "xpex.h"
#include "color.h"
#include "externs.h"
#include "globals.h"

XtAppContext app_context;
Display     *display;
Window       window;
Window       mixed_window;
Window       red_window;
Window       green_window;
Window       blue_window;
Pixmap       pixmap;
Pixmap       red_pixmap;
Pixmap       green_pixmap;
Pixmap       blue_pixmap;
GC           gc;
Visual      *visual;
int          depth;
Colormap     colormap;
Colormap     cmap;
XColor       colors[64];
GC  boxedGC, boxedChangedGC, unboxedGC;

Widget toplevel, mixingForm, mixedColor;
Widget rgb, hsv, hls, cie;
Widget hsvColor, hlsColor, cieColor;
Widget hueLocked, satLocked, valLocked;
Widget /* hueLocked, satLocked, */ lgtLocked;
Widget XLocked, YLocked, ZLocked;
Widget redLocked, greenLocked, blueLocked, title;
Widget redLabel, greenLabel, blueLabel;
Widget redIntensity, greenIntensity, blueIntensity, tripleValue;
Widget redScroll, greenScroll, blueScroll, lockedScroll;
Widget hueScroll, satScroll, valScroll;
Widget /*hueScroll, satScroll, */ lgtScroll;
Widget XScroll, YScroll, ZScroll;
Widget hueLabel, satLabel, valLabel;
Widget /* hueLabel, satLabel, */ lgtLabel;
Widget XLabel, YLabel, ZLabel;
Widget quit;

Boolean workingRGB, workingHSV, workingHLS, workingCIE;

struct app_resources {
    Boolean silent;
    String text;
    String format;
} prog_res;

static XtActionsRec actionTable[] = {
    {"lock_toggle", lock_toggle},
    {"border", border},
    /*  {"update_triple", update_triple}, */
    {"set_scroll", set_scroll},
    {"stop_scroll", stop_scroll},
    {"move_scroll", move_scroll},
};

static String fallback_resources[] = {
    NULL
};

static XrmOptionDescRec options[] = {
    { "-silent", ".silent", XrmoptionNoArg, (caddr_t)"true" },
    { "-text", ".text", XrmoptionSepArg, (caddr_t)NULL },
    { "-format", ".format", XrmoptionSepArg, (caddr_t)NULL },
};

#define offset(field) XtOffset(struct app_resources *, field)
static XtResource resources[] = {
    { "silent", "Silent", XtRBoolean, sizeof(Boolean),
    offset(silent), XtRImmediate, (XtPointer)FALSE },
    { "text", "Text", XtRString, sizeof(String),
    offset(text), XtRImmediate, (XtPointer)NULL },
    { "format", "Format", XtRString, sizeof(String),
    offset(format), XtRImmediate, (XtPointer)"#%02x%02x%02x" }
};

#undef offset


void main(argc, argv)
unsigned int argc;
char **argv;
{
    Widget top_pane, box;
    extern int PrintDefaultError();

    static XtCallbackRec callbacks[] =
    {
        {(XtCallbackProc) NULL, (XtPointer) NULL},
        {(XtCallbackProc) NULL, (XtPointer) NULL},
    };

    toplevel = XtVaAppInitialize( &app_context, "XColorEditor",
        options, XtNumber( options ),
        (int *)&argc, (String *)argv,
        fallback_resources,
        NULL );
    display = XtDisplay( toplevel );
    XSetErrorHandler( PrintDefaultError );
/*
    #include <X11/Xmu/Editres.h>
    XtAddEventHandler (toplevel, (EventMask) 0,
        True, _XEditResCheckMessages, NULL);
*/
    XtGetApplicationResources(toplevel, &prog_res,
        resources, XtNumber(resources),
        (ArgList)NULL, 0);

    XtAppAddActions(XtWidgetToApplicationContext(toplevel),
        actionTable, XtNumber(actionTable));

    if( argc > 1 ) {
        float r, g, b;
        sscanf( argv[2], "%f", &r );
        sscanf( argv[3], "%f", &g );
        sscanf( argv[4], "%f", &b );
        initColor( r, g, b );
    }
    else
        initColor( 0., 0., 0. );

    top_pane = XtVaCreateManagedWidget( "topPane",
        panedWidgetClass, toplevel, NULL );

    XtVaCreateManagedWidget( "Choose Color Model", labelWidgetClass, top_pane,
        XtNborderWidth, 0,
        NULL);

    box = XtVaCreateManagedWidget( "box", boxWidgetClass, top_pane,
        XtNorientation, XtorientHorizontal,
        XtNbackground, AllocNamedColor( "gold" ),
        NULL );

    callbacks[0].callback = (XtCallbackProc) finalQuit;
    callbacks[0].closure  = (XtPointer) NULL;
    XtVaCreateManagedWidget( "quit", commandWidgetClass, box,
        XtNcallback, callbacks,
        XtNlabel, "Quit",
        XtNbackground, AllocNamedColor( "red" ),
        NULL);

    callbacks[0].callback = (XtCallbackProc) makeRGB_Color;
    callbacks[0].closure  = (XtPointer) NULL;
    rgb = XtVaCreateManagedWidget( "rgb", commandWidgetClass, box,
        XtNcallback, callbacks,
        XtNbackground, AllocNamedColor( "gold" ),
        XtNborderWidth, 0,
        XtNlabel, "RGB",
        NULL );

    callbacks[0].callback = (XtCallbackProc) makeHSV_Color;
    callbacks[0].closure  = (XtPointer) NULL;
    callbacks[1].callback = (XtCallbackProc) createHSVWindow;
    callbacks[1].closure  = (XtPointer) NULL;
    hsv = XtVaCreateManagedWidget( "hsv", commandWidgetClass, box,
        XtNcallback, callbacks,
        XtNbackground, AllocNamedColor( "gold" ),
        XtNborderWidth, 0,
        XtNlabel, "HSV",
        NULL );

    callbacks[0].callback = (XtCallbackProc) makeHLS_Color;
    callbacks[0].closure  = (XtPointer) NULL;
    callbacks[1].callback = (XtCallbackProc) createHLSWindow;
    callbacks[1].closure  = (XtPointer) NULL;
    hls = XtVaCreateManagedWidget( "hls", commandWidgetClass, box,
        XtNcallback, callbacks,
        XtNbackground, AllocNamedColor( "gold" ),
        XtNborderWidth, 0,
        XtNlabel, "HLS",
        NULL );

    callbacks[0].callback = (XtCallbackProc) makeCIE_Color;
    callbacks[0].closure  = (XtPointer) NULL;
    callbacks[1].callback = (XtCallbackProc) createCIEWindow;
    callbacks[1].closure  = (XtPointer) NULL;
    cie = XtVaCreateManagedWidget( "cie", commandWidgetClass, box,
        XtNcallback, callbacks,
        XtNbackground, AllocNamedColor( "gold" ),
        XtNborderWidth, 0,
        XtNlabel, "CIEXYZ",
        NULL );

    XtRealizeWidget( toplevel );
    XtAppMainLoop( app_context );
}


initColor( r, g, b )
float r, g, b;
{
    unsigned long plane_masks;
    unsigned long pixels[MAX_COLORS];
    Cardinal i, j, k;

    cmap = DefaultColormap(display, DefaultScreen(display));
    if(!XAllocColorCells(display, cmap, 0, &plane_masks, 0, pixels,
        MAX_COLORS-1))
        XtError("makeColor: Not enough colormap entries available");

    /* allocate the standard cells used by the application */
    for (j = 0, i = 1; i < MEMORY_OFFSET; i++) {
        colors[i].pixel = pixels[j++];
        colors[i].flags = DoRed | DoGreen | DoBlue;
    }

    /* initialize the pixel memory locations */
    for (k = 1, i = MEMORY_OFFSET; i < MAX_COLORS; i++) {
        colors[i].flags = DoRed | DoGreen | DoBlue;
        colors[i].pixel = pixels[j++];
        colors[i].red = colors[i].green = colors[i].blue = 0;
/*      colorUsed[i-MEMORY_OFFSET] = FALSE; */
    }

    /* retrieve the given colormap entries current values */
    if (k > 1)
        XQueryColors(display, cmap, &(colors[MEMORY_OFFSET]), k-1);

    colors[MIX_COLOR].pixel = colors[MEMORY_OFFSET].pixel;
    colors[MIX_COLOR].flags = DoRed | DoGreen | DoBlue;
    colors[MIX_COLOR].red = colors[MEMORY_OFFSET].red;
    colors[MIX_COLOR].green = colors[MEMORY_OFFSET].green;
    colors[MIX_COLOR].blue = colors[MEMORY_OFFSET].blue;

    colors[MIX_COLOR].red = r * 65535;
    colors[MIX_COLOR].green = g * 65535;
    colors[MIX_COLOR].blue = b * 65535;

    colors[PURE_BLACK].red = colors[PURE_BLACK].green = 0;
    colors[PURE_BLACK].blue = 0;

    colors[PURE_RED].blue = colors[PURE_RED].green = 0;
    colors[RED_COLOR].blue = colors[RED_COLOR].green = 0;
    colors[PURE_RED].red = colors[RED_COLOR].red = (short)~0;

    /* not as pure as it should be!! looks better like this though 8-) */
    colors[PURE_GREEN].red = 0x4f*256;
    colors[PURE_GREEN].green = 0xb5*256;
    colors[PURE_GREEN].blue = 0x1e*256;

    colors[GREEN_COLOR].red = colors[GREEN_COLOR].blue = 0;
    colors[GREEN_COLOR].green = (short)~0;

    colors[PURE_BLUE].red = colors[PURE_BLUE].green = 0;
    colors[BLUE_COLOR].red = colors[BLUE_COLOR].green = 0;
    colors[PURE_BLUE].blue = colors[BLUE_COLOR].blue = (short)~0;

    XStoreColors(display, cmap, colors, MAX_COLORS);
    for (i = MEMORY_OFFSET; i < j; i++)
        colors[i].flags = DoRed | DoGreen | DoBlue;
}


void
finalQuit( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    extern void printColor();

    printColor();
    XtDestroyApplicationContext(app_context);
    (void) exit (0);
}


/* ARGSUSED */
void 
Quit(w, client, call)
Widget w;
XtPointer client, call;
{
    extern unsigned int  num_used_pixels;
    extern unsigned long *used_pixels;
    extern PEXColorSpecifier colorSpec;

    Cardinal i;
    Widget popup = XtParent( XtParent( w ) );
/*
    extern PEXRenderer        rd;
    extern PEXPipelineContext pc;
*/
    colors[MEMORY_OFFSET].red = colors[MIX_COLOR].red;
    colors[MEMORY_OFFSET].green = colors[MIX_COLOR].green;
    colors[MEMORY_OFFSET].blue = colors[MIX_COLOR].blue;
    XtReleaseGC(mixingForm, boxedGC);
    XtReleaseGC(mixingForm, boxedChangedGC);
    XtReleaseGC(mixingForm, unboxedGC);
/*
    PEXFreePipelineContext( display, pc );
    PEXFreeRenderer( display, rd );
*/
    XtPopdown( popup );
    XtUnmapWidget( popup );

    XtSetSensitive( rgb, True );    workingRGB = False;
    XtSetSensitive( hsv, True );    workingHSV = False;
    XtSetSensitive( hls, True );    workingHLS = False;
    XtSetSensitive( cie, True );    workingCIE = False;

    if( (colorSpec.type != PEXColorTypeRGB) && (depth == 8) )
    XFreeColors( display, colormap, used_pixels, num_used_pixels-1, 0 );
    num_used_pixels = 0;
}
